static char * const my_constant_names[] = {
    "FANN_TRAIN_INCREMENTAL",
    "FANN_TRAIN_BATCH",
    "FANN_TRAIN_RPROP",
    "FANN_TRAIN_QUICKPROP",
    "FANN_LINEAR",
    "FANN_THRESHOLD",
    "FANN_THRESHOLD_SYMMETRIC",
    "FANN_SIGMOID",
    "FANN_SIGMOID_STEPWISE",
    "FANN_SIGMOID_SYMMETRIC",
    "FANN_SIGMOID_SYMMETRIC_STEPWISE",
    "FANN_GAUSSIAN",
    "FANN_GAUSSIAN_SYMMETRIC",
    "FANN_GAUSSIAN_STEPWISE",
    "FANN_ELLIOT",
    "FANN_ELLIOT_SYMMETRIC",
    "FANN_LINEAR_PIECE",
    "FANN_LINEAR_PIECE_SYMMETRIC",
    "FANN_SIN_SYMMETRIC",
    "FANN_COS_SYMMETRIC",
    "FANN_SIN",
    "FANN_COS",
    "FANN_ERRORFUNC_LINEAR",
    "FANN_ERRORFUNC_TANH",
    "FANN_STOPFUNC_MSE",
    "FANN_STOPFUNC_BIT",
    0,
};
static const unsigned int my_constant_values[] = {
    FANN_TRAIN_INCREMENTAL,
    FANN_TRAIN_BATCH,
    FANN_TRAIN_RPROP,
    FANN_TRAIN_QUICKPROP,
    FANN_LINEAR,
    FANN_THRESHOLD,
    FANN_THRESHOLD_SYMMETRIC,
    FANN_SIGMOID,
    FANN_SIGMOID_STEPWISE,
    FANN_SIGMOID_SYMMETRIC,
    FANN_SIGMOID_SYMMETRIC_STEPWISE,
    FANN_GAUSSIAN,
    FANN_GAUSSIAN_SYMMETRIC,
    FANN_GAUSSIAN_STEPWISE,
    FANN_ELLIOT,
    FANN_ELLIOT_SYMMETRIC,
    FANN_LINEAR_PIECE,
    FANN_LINEAR_PIECE_SYMMETRIC,
    FANN_SIN_SYMMETRIC,
    FANN_COS_SYMMETRIC,
    FANN_SIN,
    FANN_COS,
    FANN_ERRORFUNC_LINEAR,
    FANN_ERRORFUNC_TANH,
    FANN_STOPFUNC_MSE,
    FANN_STOPFUNC_BIT,
};
